/*
   This file is part of TALER
   Copyright (C) 2026 Taler Systems SA

   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.

   TALER is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
/**
 * @file backenddb/pg_select_exchanges.c
 * @brief Implementation of the select_exchanges function for Postgres
 * @author Christian Grothoff
 */
#include "taler/platform.h"
#include <taler/taler_error_codes.h>
#include <taler/taler_dbevents.h>
#include <taler/taler_pq_lib.h>
#include "pg_select_exchanges.h"
#include "pg_helper.h"

/**
 * Context used for TMH_PG_lookup_exchanges().
 */
struct LookupExchangesContext
{
  /**
   * Function to call with the results.
   */
  TALER_MERCHANTDB_ExchangesCallback cb;

  /**
   * Closure for @a cb.
   */
  void *cb_cls;

  /**
   * Did database result extraction fail?
   */
  bool extract_failed;
};


/**
 * Function to be called with the results of a SELECT statement
 * that has returned @a num_results results about product exchanges.
 *
 * @param[in,out] cls of type `struct LookupExchangesContext *`
 * @param result the postgres result
 * @param num_results the number of results in @a result
 */
static void
lookup_exchanges_cb (void *cls,
                     PGresult *result,
                     unsigned int num_results)
{
  struct LookupExchangesContext *plc = cls;

  for (unsigned int i = 0; i < num_results; i++)
  {
    char *exchange_url;
    struct GNUNET_TIME_Absolute next_download;
    struct GNUNET_TIME_Absolute keys_expiration;
    uint32_t hc;
    uint32_t ec;
    struct GNUNET_PQ_ResultSpec rs[] = {
      GNUNET_PQ_result_spec_absolute_time ("first_retry",
                                           &next_download),
      GNUNET_PQ_result_spec_absolute_time ("expiration_time",
                                           &keys_expiration),
      GNUNET_PQ_result_spec_string ("exchange_url",
                                    &exchange_url),
      GNUNET_PQ_result_spec_uint32 ("exchange_http_status",
                                    &hc),
      GNUNET_PQ_result_spec_uint32 ("exchange_ec_code",
                                    &ec),
      GNUNET_PQ_result_spec_end
    };

    if (GNUNET_OK !=
        GNUNET_PQ_extract_result (result,
                                  rs,
                                  i))
    {
      GNUNET_break (0);
      plc->extract_failed = true;
      return;
    }
    plc->cb (plc->cb_cls,
             exchange_url,
             next_download,
             keys_expiration,
             (unsigned int) hc,
             (enum TALER_ErrorCode) ec);
    GNUNET_PQ_cleanup_result (rs);
  }
}


enum GNUNET_DB_QueryStatus
TMH_PG_select_exchanges (void *cls,
                         TALER_MERCHANTDB_ExchangesCallback cb,
                         void *cb_cls)
{
  struct PostgresClosure *pg = cls;
  struct LookupExchangesContext plc = {
    .cb = cb,
    .cb_cls = cb_cls,
    /* Can be overwritten by the lookup_exchanges_cb */
    .extract_failed = false,
  };
  struct GNUNET_PQ_QueryParam params[] = {
    GNUNET_PQ_query_param_end
  };
  enum GNUNET_DB_QueryStatus qs;

  check_connection (pg);
  PREPARE (pg,
           "select_exchanges",
           "SELECT"
           "  exchange_url"
           " ,first_retry"
           " ,expiration_time"
           " ,exchange_http_status"
           " ,exchange_ec_code"
           " FROM merchant_exchange_keys");
  qs = GNUNET_PQ_eval_prepared_multi_select (
    pg->conn,
    "select_exchanges",
    params,
    &lookup_exchanges_cb,
    &plc);
  /* If there was an error inside lookup_exchanges_cb, return a hard error. */
  if (plc.extract_failed)
  {
    GNUNET_break (0);
    return GNUNET_DB_STATUS_HARD_ERROR;
  }
  return qs;
}
