/*
  This file is part of TALER
  Copyright (C) 2014-2024 Taler Systems SA

  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU Affero General Public License as published by the Free Software
  Foundation; either version 3, or (at your option) any later version.

  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License along with
  TALER; see the file COPYING.LIB.  If not, see <http://www.gnu.org/licenses/>
*/
/**
 * @file taler_merchant_donau.h
 * @brief Structs to interact with the donau.
 * @author Bohdan Potuzhnyi
 * @author Vlada Svirsh
 */
#ifndef _TALER_MERCHANT_DONAU_H
#define _TALER_MERCHANT_DONAU_H

#include <taler/taler_util.h>
#include <taler/taler_error_codes.h>
#include <taler/taler_exchange_service.h>
#include "taler_merchant_service.h"
#include <donau/donau_service.h>
#include <donau/donau_util.h>


#include <taler/taler-merchant/get-private-donau.h>
#include <taler/taler-merchant/post-private-donau.h>
#include <taler/taler-merchant/delete-private-donau-DONAU_SERIAL.h>

#endif
