/*
   This file is part of TALER
   Copyright (C) 2026 Taler Systems SA

   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.

   TALER is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
/**
 * @file backenddb/pg_lookup_reconciliation_details.h
 * @brief implementation of the lookup_reconciliation_details function for Postgres
 * @author Christian Grothoff
 */
#ifndef PG_LOOKUP_RECONCILIATION_DETAILS_H
#define PG_LOOKUP_RECONCILIATION_DETAILS_H

#include <taler/taler_util.h>
#include <taler/taler_json_lib.h>
#include "taler/taler_merchantdb_plugin.h"


/**
 * Obtain reconciliation details for an expected incoming wire transfer.
 *
 * @param cls closure
 * @param instance_id instance to lookup payments for
 * @param expected_incoming_serial serial number of the transfer
 * @param cb callback to invoke
 * @param cb_cls closure for @a cb
 * @return transaction status code
 */
enum GNUNET_DB_QueryStatus
TMH_PG_lookup_reconciliation_details (
  void *cls,
  const char *instance_id,
  uint64_t expected_incoming_serial,
  TALER_MERCHANTDB_ReconciliationDetailsCallback cb,
  void *cb_cls);


#endif
