/*
  This file is part of TALER
  Copyright (C) 2014-2026 Taler Systems SA

  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU Lesser General Public License as published by the Free Software
  Foundation; either version 2.1, or (at your option) any later version.

  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License along with
  TALER; see the file COPYING.LGPL.  If not, see
  <http://www.gnu.org/licenses/>
*/
/**
 * @file include/taler/taler_merchant_service.h
 * @brief C interface of libtalermerchant, a C library to use merchant's HTTP API
 *        This library is not thread-safe, all APIs must only be used from a single thread.
 *        This library calls abort() if it runs out of memory. Be aware of these limitations.
 * @author Christian Grothoff
 * @author Priscilla Huang
 * @author Marcello Stanisci
 */
#ifndef _TALER_MERCHANT_SERVICE_H
#define _TALER_MERCHANT_SERVICE_H

/**
 * Version of the Taler Merchant API, in hex.
 * Thus 0.12.0-0 = 0x000C0000.
 */
#define TALER_MERCHANT_API_VERSION 0x000C0000

#include <taler/taler-merchant/common.h>
#include <taler/taler-merchant/get-config.h>
#include <taler/taler-merchant/get-management-instances.h>
#include <taler/taler-merchant/get-management-instances-INSTANCE.h>
#include <taler/taler-merchant/get-private-kyc.h>
#include <taler/taler-merchant/get-private-statistics-amount-SLUG.h>
#include <taler/taler-merchant/get-private-statistics-counter-SLUG.h>
#include <taler/taler-merchant/delete-management-instances-INSTANCE.h>
#include <taler/taler-merchant/patch-management-instances-INSTANCE.h>
#include <taler/taler-merchant/post-management-instances.h>
#include <taler/taler-merchant/post-management-instances-INSTANCE-auth.h>
#include <taler/taler-merchant/post-private-token.h>
#include <taler/taler-merchant/delete-private-tokens-SERIAL.h>
#include <taler/taler-merchant/get-private-orders.h>
#include <taler/taler-merchant/post-private-orders.h>
#include <taler/taler-merchant/get-private-orders-ORDER_ID.h>
#include <taler/taler-merchant/get-orders-ORDER_ID.h>
#include <taler/taler-merchant/delete-private-orders-ORDER_ID.h>
#include <taler/taler-merchant/post-orders-ORDER_ID-abort.h>
#include <taler/taler-merchant/post-orders-ORDER_ID-claim.h>
#include <taler/taler-merchant/post-orders-ORDER_ID-paid.h>
#include <taler/taler-merchant/post-orders-ORDER_ID-pay.h>
#include <taler/taler-merchant/post-private-orders-ORDER_ID-refund.h>
#include <taler/taler-merchant/post-orders-ORDER_ID-refund.h>
#include <taler/taler-merchant/patch-private-orders-ORDER_ID-forget.h>
#include <taler/taler-merchant/get-private-products.h>
#include <taler/taler-merchant/get-private-products-PRODUCT_ID.h>
#include <taler/taler-merchant/post-private-products.h>
#include <taler/taler-merchant/patch-private-products-PRODUCT_ID.h>
#include <taler/taler-merchant/delete-private-products-PRODUCT_ID.h>
#include <taler/taler-merchant/post-private-products-PRODUCT_ID-lock.h>
#include <taler/taler-merchant/get-products-IMAGE_HASH-image.h>
#include <taler/taler-merchant/get-private-transfers.h>
#include <taler/taler-merchant/post-private-transfers.h>
#include <taler/taler-merchant/delete-private-transfers-TID.h>
#include <taler/taler-merchant/get-private-accounts.h>
#include <taler/taler-merchant/get-private-accounts-H_WIRE.h>
#include <taler/taler-merchant/post-private-accounts.h>
#include <taler/taler-merchant/patch-private-accounts-H_WIRE.h>
#include <taler/taler-merchant/delete-private-accounts-H_WIRE.h>
#include <taler/taler-merchant/get-private-otp-devices.h>
#include <taler/taler-merchant/get-private-otp-devices-DEVICE_ID.h>
#include <taler/taler-merchant/post-private-otp-devices.h>
#include <taler/taler-merchant/patch-private-otp-devices-DEVICE_ID.h>
#include <taler/taler-merchant/delete-private-otp-devices-DEVICE_ID.h>
#include <taler/taler-merchant/get-private-templates.h>
#include <taler/taler-merchant/get-private-templates-TEMPLATE_ID.h>
#include <taler/taler-merchant/post-private-templates.h>
#include <taler/taler-merchant/patch-private-templates-TEMPLATE_ID.h>
#include <taler/taler-merchant/delete-private-templates-TEMPLATE_ID.h>
#include <taler/taler-merchant/post-templates-TEMPLATE_ID.h>
#include <taler/taler-merchant/get-templates-TEMPLATE_ID.h>
#include <taler/taler-merchant/get-private-tokenfamilies-TOKEN_FAMILY_SLUG.h>
#include <taler/taler-merchant/post-private-tokenfamilies.h>
#include <taler/taler-merchant/get-private-webhooks.h>
#include <taler/taler-merchant/get-private-webhooks-WEBHOOK_ID.h>
#include <taler/taler-merchant/post-private-webhooks.h>
#include <taler/taler-merchant/patch-private-webhooks-WEBHOOK_ID.h>
#include <taler/taler-merchant/delete-private-webhooks-WEBHOOK_ID.h>
#include <taler/taler-merchant/get-private-units.h>
#include <taler/taler-merchant/get-private-units-UNIT.h>
#include <taler/taler-merchant/post-private-units.h>
#include <taler/taler-merchant/patch-private-units-UNIT.h>
#include <taler/taler-merchant/delete-private-units-UNIT.h>
#include <taler/taler-merchant/post-private-categories.h>

#endif
