/*
   This file is part of TALER
   Copyright (C) 2026 Taler Systems SA

   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.

   TALER is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
/**
 * @file backenddb/pg_insert_unclaim_signature.h
 * @brief implementation of the insert_unclaim_signature function for Postgres
 * @author Christian Grothoff
 */
#ifndef PG_INSERT_UNCLAIM_SIGNATURE_H
#define PG_INSERT_UNCLAIM_SIGNATURE_H

#include <taler/taler_util.h>
#include <taler/taler_json_lib.h>
#include "taler/taler_merchantdb_plugin.h"

/**
 * Insert unclaim signature into DB and unclaim the order.
 *
 * @param cls closure
 * @param instance_id instance of the operation (checked)
 * @param merchant_pub public key of the instance (for notifications)
 * @param order_id identifies the order to unclaim
 * @param nonce claim nonce of the original claim
 * @param h_contract hash of the contract to unclaim
 * @param nsig signature by the nonce private key authorizing the unclaim
 * @return transaction status, 0 if an order with this @a nonce
 *   and @a h_contract was not found; 1 if the request is idempotent
 */
enum GNUNET_DB_QueryStatus
TMH_PG_insert_unclaim_signature (
  void *cls,
  const char *instance_id,
  const struct TALER_MerchantPublicKeyP *merchant_pub,
  const char *order_id,
  const struct GNUNET_CRYPTO_EddsaPublicKey *nonce,
  const struct GNUNET_HashCode *h_contract,
  const struct GNUNET_CRYPTO_EddsaSignature *nsig);

#endif
