--
-- This file is part of TALER
-- Copyright (C) 2026 Taler Systems SA
--
-- TALER is free software; you can redistribute it and/or modify it under the
-- terms of the GNU General Public License as published by the Free Software
-- Foundation; either version 3, or (at your option) any later version.
--
-- TALER is distributed in the hope that it will be useful, but WITHOUT ANY
-- WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-- A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along with
-- TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>

-- @file merchant-0031.sql
-- @brief Add field for extra meta data in wire transfer subjects
-- @author Christian Grothoff

BEGIN;

-- Check patch versioning is in place.
SELECT _v.register_patch('merchant-0031', NULL, NULL);

SET search_path TO merchant;

ALTER TABLE merchant_accounts
  ADD COLUMN extra_wire_subject_metadata TEXT DEFAULT(NULL);

COMMENT ON COLUMN merchant_accounts.extra_wire_subject_metadata
  IS 'Additional meta data that the exchange should include in wire transfer subjects made to this account on behalf of this instance';

COMMIT;
