/*
  This file is part of TALER
  (C) 2017, 2020 Taler Systems SA

  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3, or (at your option) any later version.

  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
*/
/**
 * @file taler-merchant-httpd_get-private-orders-ORDER_ID.h
 * @brief headers for GET /private/orders/ID handler
 * @author Christian Grothoff
 * @author Florian Dold
 */
#ifndef TALER_MERCHANT_HTTPD_PRIVATE_GET_ORDERS_ID_H
#define TALER_MERCHANT_HTTPD_PRIVATE_GET_ORDERS_ID_H
#include <microhttpd.h>
#include "taler-merchant-httpd.h"

/**
 * Manages a GET /private/orders/ID call, checking the status of a payment and
 * refunds and, if necessary, constructing the URL for a payment redirect URL.
 *
 * @param rh context of the handler
 * @param connection the MHD connection to handle
 * @param[in,out] hc context with further information about the request
 * @return MHD result code
 */
MHD_RESULT
TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                           struct MHD_Connection *connection,
                           struct TMH_HandlerContext *hc);


/**
 * Force resuming all long polling GET orders ID requests, we are shutting
 * down.
 */
void
TMH_force_gorc_resume (void);

#endif
